#ifndef _CONNECTION_H_
#define _CONNECTION_H_

#include <iostream>
#include <string>

#include <cstring>
#include <stdlib.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

using namespace std;

typedef struct sockaddr SocketAddress;
typedef struct sockaddr_in SocketAddressIn;
typedef unsigned char byte;

class Connection
{
	protected:
		int socketFileDesc;
		SocketAddressIn address;

		void openSocket();
		void initialize( const long, const int );

	public:
		Connection( const string&, const int );
		Connection( const long, const int );
		~Connection();

		void send( const byte*, const int, int ) const;
		void receive( byte*, const int, int ) const;
};

Connection :: Connection( const string& ip, const int port )
{
	openSocket();
	long addr = inet_addr( ip.c_str() );
	initialize( addr, port );
}

Connection :: Connection( const long ip, const int port )
{
	openSocket();
	long addr = htonl( ip );
	initialize( addr, port );
}

Connection :: ~Connection()
{
	close( socketFileDesc );
}

void Connection :: openSocket()
{
	socketFileDesc = socket( PF_INET, SOCK_STREAM, 0 );

	if( socketFileDesc == -1 )
		throw( "socket" );
}

void Connection :: initialize( const long addr, const int port )
{
	address.sin_family = AF_INET;
	address.sin_port = htons( port );
	address.sin_addr.s_addr = addr;
	memset( &(address.sin_zero), '\0', 8 );
}

void Connection :: send( const byte* message, const int messageLength, int socket = -1 ) const
{
	if( socket == -1 )
		socket = socketFileDesc;

	int bytesSent = 0;

	while( bytesSent != messageLength )
	{
		bytesSent += ::send( socket, message + bytesSent, messageLength - bytesSent, 0 );

		if( bytesSent == -1 )
			throw( "send" );
	}
}

void Connection :: receive( byte *buffer, const int messageLength, int socket = -1 ) const
{
	if( socket == -1 )
		socket = socketFileDesc;

	int bytesReceived = 0;

	while( bytesReceived != messageLength )
	{
		int received = recv( socket, buffer + bytesReceived, messageLength - bytesReceived, 0 );

		if( received == -1 || received == 0 )
			throw( "recv" );

		bytesReceived += received;
	}
}

#endif
