#!/usr/bin/perl -w

use parseConfigFile;
use createIdxFile;

use strict;

&main();

sub main()
{
	my @bucketFactors = ( 2500, 2000, 1600, 1250 );
	my $bplusTreeOrder = 100;

	my $configFile = $ARGV[0];
	my %serverInfo;
	my %langInfo;

	&parseConfigFile( $configFile, \%serverInfo, \%langInfo );

	foreach my $language ( keys( %langInfo ) )
	{
		my $origDir = $langInfo{$language}{&OND};
		my $minNgram = $langInfo{$language}{&MNI};
		my $maxNgram = $langInfo{$language}{&MXI};

		my $indexDir = $langInfo{$language}{&IND};
		my $indexDir2 = "$indexDir/Index1";
		if( !( -e $indexDir ) )
		{
			mkdir( $indexDir ) or die( "Could not create directory $indexDir!\n" );
			mkdir( $indexDir2 ) or die( "Could not create directory $indexDir2!\n" );
		}

		for( my $j = $minNgram; $j <= $maxNgram; $j++ )
		{
			my $indDir = "$indexDir2/$j" ."gms/";
			if( !( -e $indDir ) )
			{
				mkdir( $indDir ) or die( "Could not create directory $indDir!\n" );
			}

			my $origNgramDir = "$origDir/$j" . "gms/";
			my $indexPos = 0;
			print( "\nIndexing...\n\n" );
			system( "./index $bucketFactors[$j-2] $bplusTreeOrder $j $indexPos $origDir/1gms/vocab $origNgramDir $indDir" );
			print( "\nIndexing...done!\n\n" );
		}

		my $resortDir = $langInfo{$language}{&RND};
		my $isWildCard = $langInfo{$language}{&WCD};
		if( $isWildCard eq "YES" )
		{
			if( !( -e $resortDir ) )
			{
				mkdir( $resortDir ) or die( "Could not create directory $resortDir!\n" );
			}

			for( my $i = 2; $i <= $maxNgram; $i++ )
			{
				my $resortPosDir = "$resortDir/ResortPosition$i";
				if( !( -e $resortPosDir ) )
				{
					mkdir( $resortPosDir ) or die( "Could not create directory $resortPosDir!\n" );
				}

				my $indexPosDir = "$indexDir/Index$i";
				if( !( -e $indexPosDir ) )
				{
					mkdir( $indexPosDir ) or die( "Could not create directory $indexPosDir!\n" );
				}

				for( my $j = $minNgram; $j <= $maxNgram; $j++ )
				{
					if( $i > $j )
					{
						next;
					}

					my $resortNgramDir = "$resortPosDir/$j" . "gms/";
					if( !( -e $resortNgramDir ) )
					{
						mkdir( $resortNgramDir ) or die( "Could not create directory $resortNgramDir!\n" );
					}

					my $indDir = "$indexPosDir/$j" ."gms/";
					if( !( -e $indDir ) )
					{
						mkdir( $indDir ) or die( "Could not create directory $indDir!\n" );
					}

					my $origNgramDir = "$origDir/$j" . "gms/";
					my $comparePos = $i - 1;

					print( "\nSorting...\n\n" );
					system( "./psort $j $comparePos $origNgramDir $resortNgramDir" );
					print( "\nSorting...done!\n\n" );

					print( "$resortNgramDir\n" );
					print( "\nk-merging...\n\n" );
					system( "./kmerge $j $comparePos $origNgramDir $resortNgramDir" );
					print( "\nk-merging...done!\n\n" );

					system( "rm $resortNgramDir/*.sorted" );
					&createIdxFile( $resortNgramDir );

					print( "\nIndexing...\n\n" );
					system( "./index $bucketFactors[$j-2] $bplusTreeOrder $j $comparePos $origDir/1gms/vocab $resortNgramDir $indDir" );
					print( "\nIndexing...done!\n\n" );
				}
			}
		}
	}
}
