#include "keyMap.h"

KeyMap :: KeyMap( const string& file )
{
	ifstream fo( file.c_str() );

	UnigramKey key = 0;
	keyMap[SPECIAL_SYMBOL] = KeyMap::Pair( key, 0 );
	key++;
	while( !fo.eof() )
	{
		string s;
		CountInteger count;
		fo >> s >> count;

		if( fo.fail() )
			break;

		keyMap[s] = KeyMap::Pair( key, count );
		key++;
	}
	fo.close();
}

void KeyMap :: getKey( const vector<string>& words, int indexPosition, Key& key ) const
{
	UnigramKey *keyValues = new UnigramKey[words.size()];
	for( int i = 0; i < words.size(); i++ )
	{
		int pos = ( indexPosition + i ) % words.size();
		map<string, KeyMap::Pair> :: const_iterator it = keyMap.find( words[pos] );
		if( it != keyMap.end() )
			keyValues[i] = (it -> second).key;
		else
		{
			//otherwise the word is not in the vocabulary
			delete[] keyValues;
			throw( "At least one of the words is not found in the vocabulary!"  );
		}
	}
	key = Key( words.size(), keyValues );
	delete[] keyValues;
}

void KeyMap :: getKey( const Record& record, int indexPosition, Key& key ) const
{
	getKey( record.words, indexPosition, key );
}

CountInteger KeyMap :: getCount( const string& word ) const
{
	map<string, KeyMap::Pair> :: const_iterator it = keyMap.find( word );
	if( it != keyMap.end() )
		return (it -> second).count;

	//otherwise the word is not in the vocabulary
	string exception = "The word " + word + " is not found in the vocabulary during KeyMap::getCount()!";
	throw( exception.c_str() );
}
