#include <iostream>
#include <fstream>
#include <vector>
#include <queue>
#include <string>
#include <sstream>

#include <cstdlib>

using namespace std;

#include "minHeap.h"
#include "record.h"

#define MAX_RECORDS_IN_FILE	10000000
#define BUFFER_SIZE	100000

struct HeapRecord
{
	int fileNo;
	int comparePos;
	Record record;

	HeapRecord() : fileNo( -1 ), comparePos( -1 )
	{}
	HeapRecord( int fn, int cp, const Record& rec ) : fileNo( fn ), comparePos( cp ), record( rec )
	{}

	bool operator<( const HeapRecord& hr )
	{
		for( int i = comparePos; i < record.numOfWords; i++ )
		{
			short int comparison = record.words[i].compare( hr.record.words[i] );
			if( comparison < 0 )
				return true;
			if( comparison > 0 )
				return false;
		}
		for( int i = 0; i < comparePos; i++ )
		{
			short int comparison = record.words[i].compare( hr.record.words[i] );
			if( comparison < 0 )
				return true;
			if( comparison > 0 )
				return false;
		}
		return false;
	}
};

void fillBuffer( const int ngramSize, istream* fo, queue<Record>& buffer )
{
	Record dummyRecord( ngramSize );
	for( int i = 0; i < BUFFER_SIZE; i++ )
	{
		try
		{
			(*fo) >> dummyRecord;
		}
		catch( const char* cp )
		{
			break;
		}

		if( fo -> fail() )
			break;
		buffer.push( dummyRecord );
	}
}

void writeBuffer( const string& outFilePath, const vector<Record>& buffer )
{
	ofstream fo( outFilePath.c_str(), ios::app );
	for( int i = 0; i < buffer.size(); i++ )
	{
		fo << buffer[i] << endl;
	}
	fo.close();
}

string getOutFileName( const int ngramSize, int outFileCount )
{
	ostringstream oss, oss2;
	oss << ngramSize;
	oss2 << outFileCount;
	string zeroPads = "0000";
	do
	{
		zeroPads.resize( zeroPads.size() - 1 );
	}
	while( outFileCount /= 10 );
	return ( oss.str() + "gm-" + zeroPads + oss2.str() );
}

int main( int argc, char* argv[] )
{
        const int ngramSize = atoi( argv[1] );
        const int comparePos = atoi( argv[2] );

        ostringstream oss;
        oss << ngramSize;

        const string ngramFilesPath = string( argv[3] );
        const string sortedFilesPath = string( argv[4] );

	const string idxFileName = ngramFilesPath + "/" + oss.str() + "gm.idx";

	MinHeap<HeapRecord> heap;
	vector<ifstream*> fileObjects;
	vector<queue<Record> > inputBuffer;
	vector<Record> outputBuffer;
	vector<bool> isFinishedFile;
	int cnt = 0;

	ifstream fo( idxFileName.c_str() );
	while( !fo.eof() )
	{
		string fileName, firstNgram;
		fo >> fileName;
		for( int i = 0; i < ngramSize; i++ )
			fo >> firstNgram;
		if( fo.fail() )
			break;

		fileName.resize( fileName.length() - 3 );
		const string filePath = sortedFilesPath + "/" + fileName + ".sorted";
		ifstream* fileObject = new ifstream( filePath.c_str() );
		fileObjects.push_back( fileObject );
		inputBuffer.push_back( queue<Record>() );
		fillBuffer( ngramSize, fileObject, inputBuffer[cnt] );
		if( inputBuffer[cnt].size() == 0 )
			isFinishedFile.push_back( true );
		else
		{
			isFinishedFile.push_back( false );
			HeapRecord record( cnt, comparePos, inputBuffer[cnt].front() );
			heap.push( record );
			inputBuffer[cnt].pop();
		}
		cnt++;
	}
	fo.close();

	int outFileCount = 0;
	int outputCount = 0;
	while( !heap.isEmpty() )
	{
		HeapRecord record = heap.top();
		heap.pop();
		outputBuffer.push_back( record.record );
		outputCount++;
		if( outputBuffer.size() == BUFFER_SIZE )
		{
			const string outFileName = getOutFileName( ngramSize, outFileCount );
			const string outFilePath = sortedFilesPath + "/" + outFileName;
			writeBuffer( outFilePath, outputBuffer );
			outputBuffer.clear();
			if( outputCount == MAX_RECORDS_IN_FILE )
			{
				cout << "k-merged file " << outFileName << endl;
				outputCount = 0;
				outFileCount++;
			}
		}

		int fileNo = record.fileNo;

		if( isFinishedFile[fileNo] )
			continue;

		if( inputBuffer[fileNo].size() == 0 )
		{
			fillBuffer( ngramSize, fileObjects[fileNo], inputBuffer[fileNo] );
			if( inputBuffer[fileNo].size() == 0 )
			{
				isFinishedFile[fileNo] = true;
				continue;
			}
		}

		HeapRecord newRecord( fileNo, comparePos, inputBuffer[fileNo].front() );
		heap.push( newRecord );
		inputBuffer[fileNo].pop();
	}

	if( outputBuffer.size() != 0 )
	{
		const string outFileName = getOutFileName( ngramSize, outFileCount );
		const string outFilePath = sortedFilesPath + "/" + outFileName;
		writeBuffer( outFilePath, outputBuffer );
		cout << "k-merged file " << outFileName << endl;
		outFileCount++;
	}

	for( int i = 0; i < fileObjects.size(); i++ )
		delete fileObjects[i];

	return 0;
}
