#ifndef _MINHEAP_H_
#define _MINHEAP_H_

#include <vector>

using namespace std;

template <typename T>
class MinHeap
{
	private:
		vector<T> values;

		int parent( int x )
		{
			return ( ( ( x + 1 ) / 2 ) - 1 );
		}
		int left( int x )
		{
			return ( ( x * 2 ) + 1 );
		}
		int right( int x )
		{
			return ( ( x * 2 ) + 2 );
		}
		void reorderDown( int x );
		void reorderUp( int x );

	public:
		void push( const T& );
		const T& top() const;
		void pop();

		bool isEmpty() const
		{
			return ( values.size() == 0 );
		}
};


template <typename T>
void MinHeap<T> :: reorderDown( int x )
{
	if( values.size() <= 1 || x < 0 || x >= values.size() )
		return;

	int minInd = x;
	if( left( x ) < values.size() && values[left( x )] < values[x] )
		minInd = left( x );
	if( right( x ) < values.size() && values[right( x )] < values[minInd] )
		minInd = right( x );
	if( minInd != x )
	{
		T temp = values[x];
		values[x] = values[minInd];
		values[minInd] = temp;
		reorderDown( minInd );
	}
}

template <typename T>
void MinHeap<T> :: reorderUp( int x )
{
	if( values.size() <= 1 || x < 1 )
		return;

	while( parent( x ) >= 0 && values[x] < values[parent( x )] )
	{
		T temp = values[x];
		values[x] = values[parent( x )];
		values[parent( x )] = temp;
		x = parent( x );
	}
}

template <typename T>
void MinHeap<T> :: push( const T& element )
{
	values.push_back( element );
	reorderUp( values.size() - 1 );
}

template <typename T>
const T& MinHeap<T> :: top() const
{
	if( isEmpty() )
		throw "Heap is Empty!";

	return values.front();
}

template <typename T>
void MinHeap<T> :: pop()
{
	if( isEmpty() )
		throw "Heap is Empty!";

	values.front() = values.back();
	values.pop_back();
	reorderDown( 0 );
}

#endif
