#include "record.h"

Record :: Record () : numOfWords( 1 ), ngramCount( 0 )
{
	words.resize( 1 );
}

Record :: Record( int size ) : numOfWords( size ), ngramCount( 0 )
{
	words.resize( numOfWords );
}

Record :: Record( const CountInteger& count, const vector<string>& ws ) : numOfWords( ws.size() ), ngramCount( count )
{
	words.resize( numOfWords );
	for( int i = 0; i < numOfWords; i++ )
		words[i] = ws[i];
}

Record :: Record( const Record& record )
{
	*this = record;
}

bool Record :: match( const vector<string>& tokens ) const
{
	if( numOfWords != tokens.size() )
		return false;

	for( int i = 0; i < numOfWords; i++ )
	{
		if( tokens[i] != "_" && words[i] != "_" && tokens[i] != words[i] )
			return false;
	}
	return true;
}

Record& Record :: operator=( const Record& record )
{
	numOfWords = record.numOfWords;
	ngramCount = record.ngramCount;
	words.resize( numOfWords );
	for( int i = 0; i < numOfWords; i++ )
		words[i] = record.words[i];

	return *this;
}

bool Record :: operator==( const Record& record ) const
{
	if( ngramCount != record.ngramCount )
		return false;

	if( numOfWords != record.numOfWords )
		return false;

	for( int i = 0; i < numOfWords; i++ )
	{
		if( words[i] != record.words[i] )
			return false;
	}

	return true;
}

bool Record :: operator<( const Record& record ) const
{
	return ngramCount < record.ngramCount;
}

istream& operator>>( istream& is, Record& record )
{
	for( int i = 0; i < record.numOfWords; i++ )
	{
		is >> record.words[i];
		if( is.fail() )
			throw( "Couldn't read record from stream!" );
	}
	is >> record.ngramCount;

	return is;
}

ostream& operator<<( ostream& os, const Record& record )
{
	for( int i = 0; i < record.numOfWords; i++ )
	{
		os << record.words[i];
		if( i != ( record.numOfWords - 1 ) )
		{
			os << ' ';
		}
	}

	os << '\t' << record.ngramCount;

	return os;
}
